use super::*;
use crate::ctrl::net::Subnet;
use crate::ctrl::Entity;
use crate::ctrl::Storable;
use crate::ctx::Context;
use nzr_api::model;

pub async fn add_subnet(
    ctx: &Context,
    args: model::Subnet,
) -> Result<Entity<Subnet>, Box<dyn std::error::Error>> {
    let subnet = Subnet::from_model(&args.data)
        .map_err(|er| cmd_error!("Couldn't generate subnet: {}", er))?;

    let mut ent = Subnet::insert(ctx.db.clone(), subnet.clone(), args.name.as_bytes())?;

    ent.transient = true;

    if let Err(err) = ctx.zones.new_zone(&subnet).await {
        Err(cmd_error!("Failed to create new DNS zone: {}", err))
    } else {
        ent.transient = false;
        Ok(ent)
    }
}

pub fn delete_subnet(ctx: &Context, interface: &str) -> Result<(), Box<dyn std::error::Error>> {
    match Subnet::get_by_key(ctx.db.clone(), interface.as_bytes())
        .map_err(|er| cmd_error!("Couldn't find subnet: {}", er))?
    {
        Some(subnet) => subnet
            .delete()
            .map_err(|er| cmd_error!("Couldn't fully delete subnet entry: {}", er)),
        None => Err(cmd_error!("No subnet object found for {}", interface)),
    }?;

    Ok(())
}
