use std::{net::Ipv4Addr, str::FromStr};

use crate::cloud::*;
use nzr_api::net::{cidr::CidrV4, mac::MacAddr};

#[test]
fn cloud_metadata() {
    let expected = r#"
instance-id: my-instance
local-hostname: my-instance
public-keys:
- ssh-key 123456 admin@laptop
"#
    .trim_start();
    let pubkeys = vec!["ssh-key 123456 admin@laptop".to_owned(), "".to_owned()];
    let meta = Metadata::new("my-instance").ssh_pubkeys(&pubkeys);

    let meta_xml = serde_yaml::to_string(&meta).unwrap();
    assert_eq!(meta_xml, expected);
}

#[test]
fn cloud_netdata() {
    let expected = r#"
version: 2
ethernets:
  eth0:
    match:
      macaddress: 02:15:42:0b:ee:01
    addresses:
    - 192.0.2.69/24
    gateway4: 192.0.2.1
    dhcp4: false
    nameservers:
      search: []
      addresses:
      - 192.0.2.1
"#
    .trim_start();
    let mac_addr = MacAddr::new(0x02, 0x15, 0x42, 0x0b, 0xee, 0x01);
    let cidr = CidrV4::from_str("192.0.2.69/24").unwrap();
    let gateway = Ipv4Addr::from_str("192.0.2.1").unwrap();

    let dns = vec![gateway];
    let netconfig = NetworkMeta::new().static_nic(
        EtherMatch::mac_addr(&mac_addr),
        &cidr,
        &gateway,
        DNSMeta::with_addrs(None, &dns),
    );

    let net_xml = serde_yaml::to_string(&netconfig).unwrap();
    assert_eq!(net_xml, expected);
}
